/*
fuzzyn.hh

This C++ header file defines the NML Messages used for command and status by FUZZY_MODULE
Template Version 1.1

MODIFICATIONS:
5-Oct-99	Modified by rcsdesign.
21-Sep-99	Created by rcsdesign.

*/

// Prevent Multiple Inclusion
#ifndef FUZZYN_HH
#define FUZZYN_HH

// Include Files
#include "rcs.hh" 	// Common RCS definitions

// Predefined type files
// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


#define NUM_OF_FUZZY_INPUTS 2
#define NUM_OF_FUZZY_OUTPUTS 2

// Define the integer type ids.
//  RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

#define FUZZY_STATUS_TYPE 1000
#define FUZZY_HALT_TYPE 1001
#define FUZZY_INIT_TYPE 1002
#define FUZZY_RUN_TYPE 1003
#define FUZZY_SET_DEFUZZ_TYPE 1004
#define FUZZY_SET_INPUT_GAINS_TYPE 1005
#define FUZZY_SET_IN_MF_TYPE 1006
#define FUZZY_SET_IN_UD_TYPE 1007
#define FUZZY_SET_OUTPUT_GAINS_TYPE 1008
#define FUZZY_SET_OUT_MF_TYPE 1009
#define FUZZY_SET_OUT_UD_TYPE 1010
#define FUZZY_SET_PREMISE_TYPE 1011
//  RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


// Define the NML Message Classes

// Status Class
class FUZZY_STATUS : public RCS_STAT_MSG
{
public:

	// Normal Constructor
	FUZZY_STATUS();

	// Constructor used by derived classes
	FUZZY_STATUS(NMLTYPE t, size_t s) :  RCS_STAT_MSG(t,s) {};

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	double inputs_to_fuzzy[NUM_OF_FUZZY_INPUTS];
	double outputs_of_fuzzy[NUM_OF_FUZZY_OUTPUTS];

	int defuzz_type;
	int premise_type;

	// Each cycle the status of different input ud will be passed 
	int num_of_current_in_ud;
	double gain_for_in_ud;
	int num_of_current_in_mf;
	int type_of_current_in_mf;
	double center_of_current_in_mf;
	double spread_of_current_in_mf;

	// Each cycle the status of different output ud will be passed 
	int num_of_current_out_ud;
	double gain_for_out_ud;
	int num_of_current_out_mf;
	int type_of_current_out_mf;
	double center_of_current_out_mf;
	double spread_of_current_out_mf;
};

// Command Classes

class FUZZY_RUN : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_RUN();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.

};

class FUZZY_SET_IN_MF : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_SET_IN_MF();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
  int num_of_current_in_ud;
  int num_of_current_in_mf;
  int mf_type;
  double mf_center;
  double mf_spread;
  
};

class FUZZY_SET_OUT_MF : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_SET_OUT_MF();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
  int num_of_current_out_ud;
  int num_of_current_out_mf;
  int mf_type;
  double mf_center;
  double mf_spread;

};

class FUZZY_HALT : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_HALT();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.

};

class FUZZY_INIT : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_INIT();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.

};

class FUZZY_SET_DEFUZZ : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_SET_DEFUZZ();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	int defuzz_type;
};

class FUZZY_SET_INPUT_GAINS : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_SET_INPUT_GAINS();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	double input_gains[NUM_OF_FUZZY_INPUTS];
};

class FUZZY_SET_IN_UD : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_SET_IN_UD();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	int num_of_current_in_ud;
	double gain;
	double first_center;
	double last_center;
	int mf_type;
	double spread;
};

class FUZZY_SET_OUTPUT_GAINS : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_SET_OUTPUT_GAINS();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	double output_gains[NUM_OF_FUZZY_OUTPUTS];
};

class FUZZY_SET_OUT_UD : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_SET_OUT_UD();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	int num_of_current_out_ud;
	double gain;
	double first_center;
	double last_center;
	int mf_type;
	double spread;
};

class FUZZY_SET_PREMISE : public RCS_CMD_MSG
{
public:

	//Constructor
	FUZZY_SET_PREMISE();

	// CMS Update Function
	void update(CMS *);

	// Place custom variables here.
	int premise_type;
};

// Declare NML format function
extern int fuzzyFormat(NMLTYPE, void *, CMS *);

#endif 	// FUZZYN_HH
